package tvouriot.jcode.codes;

/** Classe permettant de grer l'apparition d'un caractre dans un texte.
 */
public final class Apparition implements Comparable {
    /** Caractre
     */
    private char symbole;
    /** Frquence d'apparition dans le texte
     */
    private float frequence;
    /** Code du caractre */
    private String code;
    
    /** Constructeur
     * @param c caractre
     * @param f frquence d'apparition
     */
    public Apparition(char c,float f) {
        this.symbole = c;
        this.frequence = f;
        this.code = "";
    }
    
    /** Retourne le code reprsentant le caractre
     *  @return le code
     */
    public String getCode() {
        return this.code;
    }
    
    /** Retourne le caractre
     *  @return le caractre
     */
    public char getChar() {
        return this.symbole;
    }
    
    /** Change la valeur du code
     *  @param c nouveau code
     */
    public void setCode(String c) {
        this.code = c;
    }
    
    /** Retourne la frquence d'apparition du caractre
     *  @return la frquence
     */
    public float getFreq() {
        return this.frequence;
    }
    
    /** Compare l'<CODE>Apparition</CODE> courante avec celle fournie en paramtre
     * @param a objet apparition
     * @return 1 si les deux objets sont gaux, <1 si l'objet courant est plus petit que celui pass en paramtre, >1 sinon
     */
    public int compareTo(Object a) {
        if (this.getFreq() > ((Apparition)a).getFreq())
            return 1;
        if (this.getFreq() < ((Apparition)a).getFreq())
            return -1;
        else return 0;
    }
    
    /** Retourne une chane de caractre reprsentant une Apparition
     *  @return chane
     */
    public String toString() {
        return "Char :"+this.symbole+" Freq :"+this.frequence+" Code :"+this.code;
    }
}




