package tvouriot.jcode.codes;

import java.io.*;
import java.util.*;
import javax.swing.text.*;
import javax.swing.*;

/** Classe abstraite dfinissant un codage utilis pour la compression de donnes.
 */
public abstract class CodeCompression {
    /** Nombre de caractres affiches sur une ligne de la zone de texte
     */    
    private final static byte NB_CHAR_PAR_LIGNE = 50;
    
    /** Table des correspondances entre une <CODE>Apparition</CODE> et son caractre associ
     */
    private HashMap tableChar = new HashMap();
    /** Table des correspondances entre une <CODE>Apparition</CODE> et son code associ
     */
    private HashMap tableCode = new HashMap();
    
    /** Initialisation du code  partir du texte d'un <CODE>JTextComponent</CODE>
     * @param progress barre de progression
     * @param t texte utilis pour l'initialisation
     */
    public abstract void init(JTextComponent t,JProgressBar progress);
    
    /** Initialisation du code  partir d'un fichier
     *  @param f fichier utilis pour l'initialisation
     */
    public abstract void init(File f);
    
    /** Retourne une table contenant toutes les <CODE>Apparition</CODE> d'un texte avec comme cl de recherche le caractre
     * @return la table
     */
    public final HashMap getTableChar() {
        return tableChar;
    }
    
    /** Retourne une table contenant toutes les <CODE>Apparition</CODE> d'un texte avec comme cl de recherche le code
     * @return la table
     */
    public final HashMap getTableCode() {
        return tableCode;
    }
    
    /** Fonction permettant de coder un texte
     * @param lblTaux label o sera affich le taux de compression
     * @param txt zone contenant le texte  coder
     * @param txtComp zone o le texte compress sera affich
     * @param progress barre de progression du codage
     */
    public final void coder(final JTextComponent txt, final JTextArea txtComp, final JProgressBar progress, final JLabel lblTaux) {
        
        Thread coderThread = new Thread() {
            public void run() {
                StringBuffer str = new StringBuffer(txt.getText());
                int l = str.length();
                StringBuffer strComp = new StringBuffer(l);
                Apparition a;
                int nblignes = 0;
                float taux = 0;
                
                if (l<512000) {
                    progress.setMaximum(l);
                    progress.setString("Compression");
                    
                    for (int i=0;i<l;i++) {
                        a = (Apparition)tableChar.get(new Character(str.charAt(i)));
                        strComp.append(a.getCode());
                        
                        if ((i+1)%NB_CHAR_PAR_LIGNE==0) {
                            strComp.append('\n');
                            nblignes++;
                        }
                        progress.setValue(i+1);
                    }
                    
                    txtComp.setText(strComp.toString());
                    
                    taux = (1-((float)strComp.length()-nblignes)/((float)l*8))*100;
                    lblTaux.setText("Taux de compression : "+taux+"%");
                }
                else txtComp.setText("Le texte  compresser doit avoir une taille infrieure  500 Ko !");
                
                progress.setString("Compression termine");
            }
        };
        coderThread.start();
    }
    
    /** Fonction permettant de coder un texte
     *  @param src fichier  coder
     *  @param dst fichier o sera enregistr le code
     */
    public final void coder(File src, File dst) {
        
        
        
        
    }
    
    /** Fonction permettant de dcoder un texte cod
     *  @param txt zone contenant le texte  dcoder et o sera affich le rsultat
     *  @param progress barre de progression du dcodage
     */
    public final void decoder(final JTextComponent txt, final JProgressBar progress) {
        Thread decoderThread = new Thread() {
            public void run() {
                StringBuffer str = new StringBuffer(txt.getText());
                int l = str.length();
                StringBuffer strDecomp = new StringBuffer();
                Apparition a;
                StringBuffer code = new StringBuffer();
                int c = 0;
                progress.setMaximum(l);
                progress.setString("Dcompression");
                
                for (int i=0;i<l;i++) {
                    code.append(str.charAt(i));
                    a = (Apparition)tableCode.get(code.toString());
                    if (a != null) {
                        strDecomp.append(a.getChar());
                        code.setLength(0);
                        c++;
                        if (c%NB_CHAR_PAR_LIGNE==0)
                            i++;
                    }
                    progress.setValue(i+1);
                }
                
                txt.setText(strDecomp.toString());
                progress.setString("Dcompression termine");
            }
        };
        decoderThread.start();
    }
    
    /** Fonction permettant de dcoder un texte cod
     *  @param src fichier contenant le code
     *  @param dst fichier dans lequel se fera le dcodage
     */
    public final void decoder(File src, File dst) {
        
        
        
        
    }
    
    /** Fonction retournant la longueur moyenne du code
     *  @return longueur moyenne
     */
    public float getLgMoy() {
        Object [] values = tableCode.values().toArray();
        float lgMoy = 0;
        Apparition a;
        
        for (int i=0;i<values.length;i++) {
            a = (Apparition)values[i];
            lgMoy += a.getCode().length() * a.getFreq();
        }
        return lgMoy;
    }
}