package tvouriot.jcode.codes;

import java.util.*;
import java.io.*;
import javax.swing.text.*;
import javax.swing.*;
import tvouriot.lib.arbres.*;

/** Classe implantant le code de compression de Huffman.
 */
public final class Huffman extends CodeCompression {
    /** Constructeur
     * @param progress barre de progression
     * @param t texte composant contenant le texte  compresser
     */
    public Huffman(JTextComponent t,JProgressBar progress) {
        this.init(t,progress);
    }
    
    /** Initialisation du code  partir du texte d'un JTextComponent
     * @param textZone texte  traiter
     * @param progress barre de progression
     */
    public void init(JTextComponent textZone,JProgressBar progress) {
        Vector list = new Vector();
        String text = textZone.getText();
        float l = (float)text.length();
        
        
        int [] ascii = new int[65536];
        for (int i=0;i<l;i++) {
            ascii[(int)text.charAt(i)]++;
        }
        for (int i=0;i<65536;i++) {
            if (ascii[i] != 0)
                list.addElement(new ArbinObjCh(new Apparition((char)i,(float)ascii[i] / l)));
        }
        progress.setMaximum(list.size());
        progress.setString("Init table");
        huffman(list,getTableChar(),getTableCode(),progress);
        progress.setString("Init table termine");
    }
    
    /** Initialisation du code  partir d'un fichier
     *  @param f fichier utilis pour l'initialisation
     */
    public void init(File f) {
        
    }
    
    /** Fonction appliquant l'algorithme de Huffman
     * @param list liste des <CODE>Apparition</CODE>
     * @param tableChar premire table  remplir
     * @param tableCode seconde table  remplir
     * @param progress barre de progression
     */
    private static void huffman(Vector list, HashMap tableChar, HashMap tableCode, JProgressBar progress) {
        ArbinObjCh premier;
        ArbinObjCh second;
        float f1;
        float f2;
        Apparition noeud;
        while (list.size() > 1) {
            list = trier(list);
            premier = (ArbinObjCh)list.get(0);
            second = (ArbinObjCh)list.get(1);
            
            f1 = ((Apparition)premier.racine()).getFreq();
            f2 = ((Apparition)second.racine()).getFreq();
            
            noeud = new Apparition('#',f1+f2);
            list.setElementAt(new ArbinObjCh(noeud,premier,second),0);
            list.removeElementAt(1);
            
            progress.setValue(progress.getMaximum()-list.size());
        }
        genererCode((ArbinObj)list.get(0));
        genererTables((ArbinObj)list.get(0),tableChar,tableCode);
        
        progress.setValue(progress.getMaximum());
    }
    
    /** Fonction permettant de trier un vecteur d'<CODE>Apparition</CODE>
     * @param list liste  trier
     * @return liste trie
     */
    private static Vector trier(Vector list) {
        Vector l = new Vector();
        int min,c;
        
        for (int i = 0;i<list.size();) {
            c = i;
            min = c;
            for (;c<list.size();c++) {
                //on recherche le min
                ArbinObjCh a = (ArbinObjCh)list.get(c);
                Apparition n = (Apparition)a.racine();
                ArbinObjCh a1 = (ArbinObjCh)list.get(min);
                Apparition n1= (Apparition)a1.racine();
                
                if (n.compareTo(n1)<0)
                    min = c;
            }
            //on permute le min avec l'elem d'indice i)
            l.addElement((ArbinObjCh)list.elementAt(min));
            list.remove(min);
        }
        return l;
    }
    
    /** Fonction gnrant les codes de chaques caractres
     * @param arb arbre contenant les <CODE>Apparition</CODE>
     */
    private static void genererCode(ArbinObj arb) {
        if (!arb.ag().avide()) {
            ((Apparition)arb.ag().racine()).setCode(((Apparition)arb.racine()).getCode()+"0");
            genererCode(arb.ag());
        }
        
        if (!arb.ad().avide()) {
            ((Apparition)arb.ad().racine()).setCode(((Apparition)arb.racine()).getCode()+"1");
            genererCode(arb.ad());
        }
    }
    
    /** Fonction remplissant les diffrentes tables
     * @param arb arbre contenant les <CODE>Apparition</CODE>
     * @param tableChar premire table  remplir
     * @param tableCode seconde table  remplir
     */
    private static void genererTables(ArbinObj arb, HashMap tableChar, HashMap tableCode) {
        if (arb.ag().avide() && arb.ad().avide()) {
            Apparition a = (Apparition)arb.racine();
            tableChar.put(new Character(a.getChar()),a);
            tableCode.put(a.getCode(),a);
        }
        else {
            if (!arb.ad().avide())
                genererTables(arb.ad(),tableChar,tableCode);
            if (!arb.ag().avide())
                genererTables(arb.ag(),tableChar,tableCode);
        }
        
    }
}
