package tvouriot.jcode.gui;

import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import tvouriot.jcode.codes.*;
import tvouriot.lib.io.*;

/** Panel contenant les composants de l'application.
 */
public final class PnlCompVirtuelle extends JPanel {
    private JPanel pnlTable = new JPanel(new BorderLayout());
    private JPanel pnlTxt = new JPanel(new BorderLayout());
    private JPanel pnlTxtComp = new JPanel(new BorderLayout());
    private JLabel lblInfosFichier = new JLabel(" ");
    private JLabel lblNbChar = new JLabel();
    private JLabel lblLgMoy = new JLabel();
    private JLabel lblTaux = new JLabel();
    private JTextArea txt = new JTextArea(10,1);
    private JTextArea txtComp = new JTextArea();
    private DefaultTableModel tableModel = new DefaultTableModel();
    private JTable table = new JTable(tableModel);
    private JFileChooser fc = new JFileChooser(this.getClass().getResource("PnlCompVirtuelle.class").toString());
    private JProgressBar progress = new JProgressBar();
    private CodeCompression code;
    private MainToolBar tb;
    
    /** Constructeur
     *  @param toolbar barre d'outils de l'application
     */
    public PnlCompVirtuelle(MainToolBar toolbar) {
        tb = toolbar;
        this.setLayout(new BorderLayout());
        
        Font font = new Font("Dialog",Font.PLAIN,11);
        lblInfosFichier.setFont(font);
        lblNbChar.setFont(font);
        lblLgMoy.setFont(font);
        lblTaux.setFont(font);
        
        JPanel pnlInfosFichier = new JPanel(new FlowLayout(FlowLayout.RIGHT,20,5));
        JPanel pnlTaux = new JPanel(new FlowLayout(FlowLayout.RIGHT,20,5));
        JPanel pnlProgress = new JPanel(new FlowLayout(FlowLayout.LEFT,20,4));
        JPanel pnlInfosTable = new JPanel(new BorderLayout());
        JPanel pnlSouth = new JPanel(new GridLayout(1,2));
        
        pnlInfosFichier.add(lblInfosFichier);
        
        pnlTxt.add(new JLabel(" Texte :"),BorderLayout.NORTH);
        pnlTxt.add(new JScrollPane(txt));
        pnlTxt.add(pnlInfosFichier,BorderLayout.SOUTH);
        
        progress.setFont(font);
        progress.setMinimum(0);
        progress.setStringPainted(true);
        progress.setBorderPainted(true);
        progress.setString("");
        pnlProgress.add(new JLabel("Etat :"));
        pnlProgress.add(progress);
        pnlTaux.add(lblTaux);
        pnlSouth.add(pnlProgress);
        pnlSouth.add(pnlTaux);
        
        txtComp.setBackground(Color.lightGray);
        pnlTxtComp.add(new JLabel(" Texte compress :"),BorderLayout.NORTH);
        pnlTxtComp.add(new JScrollPane(txtComp));
        pnlTxtComp.add(pnlSouth,BorderLayout.SOUTH);
        
        pnlInfosTable.add(lblNbChar,BorderLayout.NORTH);
        pnlInfosTable.add(lblLgMoy,BorderLayout.SOUTH);
        
        pnlTable.add(new JLabel(" Table des correspondances :"),BorderLayout.NORTH);
        pnlTable.add(new JScrollPane(table));
        pnlTable.add(pnlInfosTable,BorderLayout.SOUTH);
        
        JSplitPane splitH = new JSplitPane(JSplitPane.VERTICAL_SPLIT,pnlTxt,pnlTxtComp);
        splitH.setOneTouchExpandable(true);
        JSplitPane splitV = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,pnlTable,splitH);
        splitV.setOneTouchExpandable(true);
        this.add(splitV);
        
        txt.getDocument().addDocumentListener(new DocumentListener() {
            public void removeUpdate(javax.swing.event.DocumentEvent documentEvent) {
                tb.cmdInitTable.setEnabled(true);
            }
            
            public void insertUpdate(javax.swing.event.DocumentEvent documentEvent) {
                tb.cmdInitTable.setEnabled(true);
            }
            
            public void changedUpdate(javax.swing.event.DocumentEvent documentEvent) {
            }
        });
        
        tb.cmdCompresser.setBorderPainted(false);
        tb.cmdDecompresser.setBorderPainted(false);
        tb.cmdInitTable.setBorderPainted(false);
        tb.cmdCompresser.setEnabled(false);
        tb.cmdDecompresser.setEnabled(false);
        tb.cmdInitTable.setEnabled(false);
    }
    
    /** Fonction permettant l'ouverture d'un fichier */
    public void openFile() {
        if (fc.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
            effacerAll();
            File f = fc.getSelectedFile();
            Fichiers.openFile(txt,f);
            String infos = f.toString()+"   "+f.length()+" octets";
            lblInfosFichier.setText(infos);
            
            tb.cmdInitTable.setEnabled(true);
        }
    }
    
    /** Fonction permettant l'initialisation de la table des correspondances */
    public void initTable() {
        code = new Huffman(txt,progress);
        Apparition a;
        Object [] tab = code.getTableChar().values().toArray();
        Arrays.sort(tab);
        
        Object [][] values = new Object[tab.length][4];
        for (int i=0;i<tab.length;i++) {
            a = (Apparition)tab[i];
            values[i][0] = new Character(a.getChar());
            values[i][1] = new Integer(a.getChar());
            values[i][2] = new Float(a.getFreq());
            values[i][3] = a.getCode();
        }
        String [] colonnes = {"Caractre","ASCII","Frquence","Code"};
        tableModel = new DefaultTableModel(values,colonnes);
        
        table.setModel(tableModel);
        this.lblNbChar.setText(" Nbre de caractres diffrents : "+values.length);
        this.lblLgMoy.setText(" Longueur moyenne du code : "+code.getLgMoy());
        
        tb.cmdCompresser.setEnabled(true);
        tb.cmdInitTable.setBorderPainted(false);
        tb.cmdInitTable.setEnabled(false);
    }
    
    /** Permet de sauver dans un fichier le contenu de la zone texte */
    public void save() {
        JOptionPane DialogBox = new JOptionPane("Info",JOptionPane.ERROR_MESSAGE);
        if (fc.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) {
            File f = fc.getSelectedFile();
            String s = "Le fichier slectionn existe dj !\n";
            s += "Voulez-vous vraiment l'crasez ?\n";
            if ((!f.exists()) || (DialogBox.showConfirmDialog(this,s,"Attention",JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION)) {
                Fichiers.saveFile(txt,f);
                String infos = f.toString()+"   "+f.length()+" octets";
                lblInfosFichier.setText(infos);
            }
        }
    }
    
    /** Fonction permettant de lancer le codage */
    public void coder() {
        code.coder(txt,txtComp,progress,lblTaux);
        tb.cmdCompresser.setBorderPainted(false);
        tb.cmdCompresser.setEnabled(false);
        tb.cmdDecompresser.setEnabled(true);
    }
    
    /** Fonction permettant de lancer le dcodage */
    public void decoder() {
        code.decoder(txtComp,progress);
        lblTaux.setText("");
        tb.cmdDecompresser.setBorderPainted(false);
        tb.cmdCompresser.setEnabled(true);
        tb.cmdDecompresser.setEnabled(false);
    }
    
    /** Fonction permettant d'effacer toutes les zones de l'application */
    public void effacerAll() {
        this.txt.setText("");
        this.txtComp.setText("");
        this.lblInfosFichier.setText(" ");
        this.lblLgMoy.setText("");
        this.lblNbChar.setText("");
        this.lblTaux.setText("");
        this.progress.setString("");
        this.tableModel = new DefaultTableModel();
        this.table.setModel(tableModel);
        code = null;
        
        tb.cmdCompresser.setBorderPainted(false);
        tb.cmdDecompresser.setBorderPainted(false);
        tb.cmdInitTable.setBorderPainted(false);
        tb.cmdCompresser.setEnabled(false);
        tb.cmdDecompresser.setEnabled(false);
        tb.cmdInitTable.setEnabled(false);
    }
}
