package tvouriot.lib.arbres;

/** Classe implantant le type "arbre chain d'objets". */
public class ArbinObjCh extends ArbinObj {
    /** Classe reprsentant un noeud d'un arbre
     */
    private static final class Noeud {
        /** Valeur du noeud
         */
        Object valeur;
        /** Fils gauche
         */
        Noeud fg;
        /** Fils droit
         */
        Noeud fd;
        
        /** Constructeur unique
         */
        public Noeud() {
            valeur = null;
            fg=fd=null;
        }
    }
    
    /** Racine de l'arbre courant */
    private Noeud rac;
    
    /** Constructeur qui instancie un arbre vide */
    public ArbinObjCh() {
        this.rac = new Noeud();
    }
    
    /** Constructeur
     *  @param x racine
     */
    public ArbinObjCh(Object x) {
        this();
        this.rac.valeur = x;
    }
    
    /** Constructeur
     * @param x racine
     * @param g arbre gauche
     * @param d arbre droit
     */
    public ArbinObjCh(Object x, ArbinObjCh g,ArbinObjCh d) {
        this(x);
        this.rac.fg = g.rac;
        this.rac.fd = d.rac;
    }
    
    /** Cr un arbre avec une racine, un arbre gauche et un arbre droit
     *  @param x racine
     *  @param g arbre gauche
     *  @param d arbre droit
     *  @return arbre cr
     */
    protected ArbinObj creationArbre(Object x,ArbinObj g,ArbinObj d) {
        return new ArbinObjCh(x,(ArbinObjCh) g, (ArbinObjCh) d);
    }
    
    /** Cr un arbre avec une racine
     *  @param x racine
     *  @return arbre cr
     */
    protected ArbinObj creationArbre(Object x) {
        return new ArbinObjCh(x);
    }
    
    /** Teste si l'arbre est vide
     * @return <CODE>true</CODE> si l'arbre est vide, <CODE>false</CODE> sinon
     */
    public boolean avide() {
        return this.rac == null;
    }
    
    /** Transforme l'arbre courant en un arbre vide */
    public void setArbreVide() {
        this.rac = null;
    }
    
    /** Retourne un arbre vide
     *  @return arbre vide
     */
    public ArbinObj getArbreVide() {
        ArbinObjCh a = null;
        return a;
    }
    
    /** Retourne la racine de l'arbre courant
     * @return la racine
     */
    public Object racine() {
        return this.rac.valeur;
    }
    
    /** Retourne l'arbre gauche de l'arbre courant
     *  @return l'arbre gauche
     */
    public ArbinObj ag() {
        ArbinObjCh arbre = new ArbinObjCh() ;
        arbre.rac =  this.rac.fg ;
        return arbre ;
    }
    
    /** Retourne l'arbre droit de l'arbre courant
     *  @return l'arbre droit
     */
    public ArbinObj ad() {
        ArbinObjCh arbre = new ArbinObjCh() ;
        arbre.rac = this.rac.fd ;
        return arbre ;
    }
}
