package tvouriot.lib.gui;

import java.awt.*;
import java.net.*;

/** Classe qui permet d'afficher une image d'introduction (sans cadre) au milieu de l'cran pendant un certain laps de temps.
 */
public class SplashScreen extends Window implements Runnable {
    /** Image  afficher
     */    
    private Image image;
    /** Boolen permettant de savoir si l'affichage doit se terminer
     */    
    private boolean terminated = false;
    
    /** Constructeur unique
     * @param img URL de l'image  afficher
     */
    public SplashScreen(URL img) {
        super(new Frame());
        
        // Load image and wait to complete
        image = Toolkit.getDefaultToolkit().getImage(img);
        MediaTracker tracker = new MediaTracker(this);
        tracker.addImage(image, 0);
        try {
            tracker.waitForID(0);
        }
        catch (Exception e) {}
        
        // Set window size and position
        int w = image.getWidth(this);
        int h = image.getHeight(this);
        setSize(w, h);
        Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
        setLocation((screen.width - w)/2, (screen.height - h)/2);
        show();
        new Thread(this).start();
    }
    
    /** Termine l'affichage de l'image
     */
    public synchronized void setTerminated() {
        terminated = true;
        setVisible(false);
        notify();
    }
    
    /** Affiche l'image  l'cran
     * @param g Contexte graphique
     */
    public synchronized void paint(Graphics g) {
        g.drawImage(image, 0, 0, this);
        notify();
    }
    
    /** Affiche l'image tant que setTerminated n'est pas appel */
    public synchronized void run() {
        while (!terminated) {
            try { wait(); }
            catch (InterruptedException e) {}
        }
    }
}
