package tvouriot.lib.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

/** Classe contenant des outils pour l'interface graphique <I>(Swing)</I>.
 */
public class SwingTools {
    
    /** Look and feel "Metal" spcialement cre pour Java */
    public final static int METAL = 1;
    /** Look and feel "Motif" employ sur de trs nombreuses stations UNIX */
    public final static int MOTIF = 2;
    /** Look and feel "Windows" de l'OS de Microsoft */
    public final static int WINDOWS = 3;
    /** Look and feel "Macintosh" de l'OS d'Apple */
    public final static int MACINTOSH = 4;
    
    /** Fonction permettant de faciliter la cration de menu
     * @param parent <CODE>String</CODE> ou <CODE>JMenu</CODE>
     * @param items tableau contenant des <CODE>String</CODE> ou des <CODE>JMenuItem</CODE>
     * @param target objet qui s'occupe de la gestion des clicks sur le menu
     * @return <CODE>JMenu</CODE> rsultant
     */
    public static JMenu makeMenu(Object parent, Object [] items,Object target) {
        JMenu m=null;
        
        if(parent instanceof JMenu)
            m=(JMenu)parent;
        else if (parent instanceof String)
            m=new JMenu((String)parent);
        else
            return null;
        
        for (int i=0;i<items.length;i++) {
            if(items[i]==null)
                m.addSeparator();
            else
                m.add(makeMenuItem(items[i],target));
        }
        return m;
    }
    
    /** Fonction qui fabrique un <CODE>JMenuItem</CODE> avec son <I>listener</I>
     * @param item <CODE>String</CODE> ou <CODE>JMenu</CODE>
     * @param target objet qui s'occupe de la gestion des clicks de l'item
     * @return <CODE>JMenuItem</CODE> rsultant
     */
    public static JMenuItem makeMenuItem(Object item,Object target) {
        JMenuItem r=null;
        
        if (item instanceof String)
            r=new JMenuItem((String)item);
        else if(item instanceof JMenuItem)
            r=(JMenuItem)item;
        else
            return null;
        
        if (target instanceof ActionListener)
            r.addActionListener((ActionListener)target);
        
        return r;
    }
    
    /** Fonction permettant de faciliter la cration de popup menu
     * @param items tableau contenant des <CODE>String</CODE> ou des <CODE>JMenuItem</CODE>
     * @param target objet qui s'occupe de la gestion des clicks sur le menu
     * @return <CODE>JPopupMenu</CODE> rsultant
     */
    public static JPopupMenu makePopupMenu(Object [] items,Object target) {
        JPopupMenu m=new JPopupMenu();
        
        for(int i=0;i<items.length;i++) {
            if (items[i]==null)
                m.addSeparator();
            else
                m.add(makeMenuItem(items[i],target));
        }
        return m;
    }
    
    /** Permet de changer le <I>look & feel</I> de l'interface
     * @param fr frame dont le look doit tre chang
     * @param plaf numro du <I>look & feel</I> (voir constantes)
     */
    public static void changePlaf(JFrame fr, int plaf) {
        JOptionPane DialogBox = new JOptionPane("Info",JOptionPane.ERROR_MESSAGE);
        try {
            switch (plaf) {
                case 1:
                    UIManager.setLookAndFeel
                    ("javax.swing.plaf.metal.MetalLookAndFeel");
                    break;
                case 2:
                    UIManager.setLookAndFeel
                    ("com.sun.java.swing.plaf.motif.MotifLookAndFeel");
                    break;
                case 3:
                    UIManager.setLookAndFeel
                    ("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
                    break;
                case 4:
                    UIManager.setLookAndFeel
                    ("javax.swing.plaf.mac.MacLookAndFeel");
                    break;
            }
            SwingUtilities.updateComponentTreeUI(fr);
        }
        catch (UnsupportedLookAndFeelException e) {
            String s = "Look & Feel non disponible sur votre systme.\n";
            if (plaf==3)
                s += "Ce look n'est disponible que sur une plate-forme Windows.";
            else if (plaf==4)
                s += "Ce look n'est disponible que sur une plate-forme Macintosh.";
            DialogBox.showMessageDialog(fr,s,"Attention",JOptionPane.INFORMATION_MESSAGE);
        }
        catch (IllegalAccessException e) {
            String s = "Look & Feel non accessible sur votre systme.\n";
            if (plaf==3)
                s += "Ce look n'est disponible que sur une plate-forme Windows.";
            else if (plaf==4)
                s += "Ce look n'est disponible que sur une plate-forme Macintosh.";
            DialogBox.showMessageDialog(fr,s,"Attention",JOptionPane.INFORMATION_MESSAGE);
        }
        catch (ClassNotFoundException e) {
            String s = "Look & Feel non trouv.\n";
            if (plaf==3)
                s += "Ce look n'est disponible que sur une plate-forme Windows.";
            else if (plaf==4)
                s += "Ce look n'est disponible que sur une plate-forme Macintosh.";
            DialogBox.showMessageDialog(fr,s,"Attention",JOptionPane.INFORMATION_MESSAGE);
        }
        catch (InstantiationException e) {
            System.out.println
            ("Ce Look & Feel ne peut tre instanci.");
        }
        catch (Exception e) {
            System.out.println("Erreur d'excution.");
        }
    }
    
    /** Renvoie les coordonnes pour que la seconde fentre soit positionne au milieu de la premire
     * @param f1 premire fentre
     * @param f2 seconde fentre
     * @return coordonnes
     */
    public static Point getPositionCentrale(Window f1, Window f2) {
        return new Point(f1.getX()+f1.getWidth()/2-f2.getWidth()/2,f1.getY()+f1.getHeight()/2-f2.getHeight()/2);
    }
    
    /** Renvoie les coordonnes pour que la seconde fentre soit positionne au milieu de la premire
     * @param f1 premire fentre
     * @param f2 seconde fentre
     * @return coordonnes
     */
    public static Point getPositionCentrale(JInternalFrame f1, Window f2) {
        Point p=f1.getLocationOnScreen();
        return new Point(p.x+f1.getWidth()/2-f2.getWidth()/2,p.y+f1.getHeight()/2-f2.getHeight()/2);
    }
    
    /** Renvoie les coordonnes pour que la fentre soit au centre de l'cran
     * @param w fentre  placer au centre de l'cran
     * @return coordonnes
     */    
    public static Point getPositionCentrale(Window w) {
        Dimension reso = w.getToolkit().getScreenSize();
        return new Point(reso.width/2-w.getWidth()/2,reso.height/2-w.getHeight()/2);
    }
}